/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.workers;

import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.AlignmentI;
import jalview.gui.AlignFrame;
import jalview.gui.AlignmentPanel;
import jalview.gui.FeatureRenderer;

import java.util.ArrayList;
import java.util.List;

/**
 * A class to create and update one or more alignment annotations, given a
 * 'calculator'.
 * 
 */
class AnnotationWorker extends AlignCalcWorker
{
  /*
   * the provider of the annotation calculations
   */
  AnnotationProviderI counter;

  /**
   * Constructor
   * 
   * @param af
   * @param counter
   */
  public AnnotationWorker(AlignFrame af, AnnotationProviderI counter)
  {
    super(af.getViewport(), af.alignPanel);
    ourAnnots = new ArrayList<AlignmentAnnotation>();
    this.counter = counter;
    calcMan.registerWorker(this);
  }

  @Override
  public void run()
  {
    try
    {
      calcMan.notifyStart(this);

      while (!calcMan.notifyWorking(this))
      {
        try
        {
          Thread.sleep(200);
        } catch (InterruptedException ex)
        {
          ex.printStackTrace();
        }
      }
      if (alignViewport.isClosed())
      {
        abortAndDestroy();
        return;
      }

      removeAnnotations();
      AlignmentI alignment = alignViewport.getAlignment();
      if (alignment != null)
      {
        try
        {
          List<AlignmentAnnotation> anns = counter.calculateAnnotation(
                  alignment, new FeatureRenderer((AlignmentPanel) ap));
          for (AlignmentAnnotation ann : anns)
          {
            ann.showAllColLabels = true;
            ann.graph = AlignmentAnnotation.BAR_GRAPH;
            ourAnnots.add(ann);
            alignment.addAnnotation(ann);
          }
        } catch (IndexOutOfBoundsException x)
        {
          // probable race condition. just finish and return without any fuss.
          return;
        }
      }
    } catch (OutOfMemoryError error)
    {
      ap.raiseOOMWarning("calculating annotations", error);
      calcMan.workerCannotRun(this);
    } finally
    {
      calcMan.workerComplete(this);
    }

    if (ap != null)
    {
      ap.adjustAnnotationHeight();
      ap.paintAlignment(true);
    }

  }

  /**
   * Remove all our annotations before re-calculating them
   */
  void removeAnnotations()
  {
    for (AlignmentAnnotation ann : ourAnnots)
    {
      alignViewport.getAlignment().deleteAnnotation(ann);
    }
    ourAnnots.clear();
  }

  @Override
  public void updateAnnotation()
  {
    // do nothing
  }
}
